function C = colorCircle(T,bg)
% colorCircle  Create a square matrix with an embedded circular temperature profile
%              The radial temperature profile is specifed by the input vector T.
%              This profile is used to generate the temperature/color map.
%              The size (n by n) of the matrix is determined by the length of T.
%
% Synopsis:   C = colorCircle(T,bg)
%
% Input:      T = vector of radial temperature values
%             bg = background value (temperature)
%
% Output:     C = square matrix with values between min(T) and max(T)

n = length(T);  nn = 2*n;

% --- Create the kr index matrix such that kr(i,j) gives the
%     index in the radial temperature vector corresponding to
%     the Cartesian position x(j),y(n-i).
%     Roughly, kr(i,j) = sqrt( j^2 + (n-i)^2 )
%
%     Use vectorized calculations to compute kr.  First construct
%     the I and J matrices which contain duplicate rows and columns
%     of the i and j indices.  Then use a one-liner to compute kr

% [I,J] = meshgrid(1:n,n:-1:1);
% kr = sqrt( I.^2 + J.^2 );
% I = [];  J = [];            %  Dispose of I and J to save memory
kr = sqrt( repmat(1:n,n,1).^2 + repmat((n:-1:1)',1,n).^2 );

% --- To simultaneously set the background value create Taug, the augmented
%     T array, which is one element larger than T.  The extra element has
%     a temperature value equal to the background
Taug = [T(:); bg];

% --- Following statement filters kr values such that no entry in
%     the kr matrix is greater than n+1.  Since the kr elements are used
%     as indices into the T vector we (normally would) want no element of
%     kr to have a value greater than n = length(T).  However we are going
%     to use Taug, which has length n+1.  Here are the pieces of the following
%     on-liner:
%
%          kr>n is an n by n matrix of ones and zeros.  The ones are in positions
%          where kr(i,j)>n is true.
%
%          kr<=n is an n by n matrix of ones and zeros.  The ones are in positions
%          where kr(i,j)<=n is true.
%
%          kr.*(kr<=n) is the array product of the kr matrix and the (kr<=n) matrix.
%          The result is a matrix that has zeros in any position of the orignal
%          kr matrix where kr(i,j)<=n is true.  The other elements of kr.*(kr<=n)
%          have the original kr(i,j) values.
%
%          Adding (n+1)*(kr>n) to kr.*(kr<=n) produces a matrix where all elements
%          of the original kr matrix having kr(i,j)>n are set to the value of n+1.

kr = (n+1)*(kr>n) + kr.*(kr<=n);
kr = round(kr);                      %  make sure kr values are integers

%  --- NOW, assigning the elements of C4 is a trivial exercise
C4 = Taug(kr);
C4 = reshape(C4,n,n);

% --- Create C by reflecting C4 into other quadrants
C = bg*ones(n,n);
C(1:n,n+1:nn)    = C4;                     %  upper right quadrant
C(1:n,1:n)       = fliplr(C4);             %  upper left quadrant
C(n+1:nn,1:n)    = fliplr(flipud(C4));     %  lower left quadrant
C(n+1:nn,n+1:nn) = flipud(C4);             %  lower right quadrant

end
