function theMovie = oneSphere(ntstep,fastRender,cmap)
% oneSphere  Animation of transient conduction in a sphere with convective BC
%            The temperature distribution is from the analytical solution
%            for Biot number of 10.  Memory consumption increases with the
%            number of time steps in the movie.  Reduce the number of time
%            steps if you run out of memory.  The number of time steps has
%            no effect on the accuracy of the results since the analytical
%            solution is used to compute T(r,t) at any time, t.
%
%            To make an AVI file from this animation, use the makeSphereMovie
%            function inclu
%
% Synopsis:  oneSphere
%            oneSphere(n)
%            oneSphere(n,fastrender)
%            oneSphere(n,fastrender,cmap)
%            M = oneSphere(...)
%
% Input:     n = number of time steps.  Default: n = 45
%            fastRender = flag used to select how the temperature image
%                         is created from the temperature field values
%                         fastRender = true, use pixel replication to fill
%                         in the color image
%                         fastRender = false, use bilinear interpolation
%                         to give smooth image
%                         Default: fastrender = true
%            cmap = (string) name of colormap used to represent temperature
%                   Default:  cmap = 'jet'
%            
%
% Output:    M = (optional) MATLAB movie matrix.  Default is to play the
%                movie twice without returning it.

%  Provide defaults for optional input
if nargin<1, ntstep = 45;        end
if nargin<2, fastRender = true;  end
if nargin<3, cmap='jet';         end

% -- Image size is hard-coded:  nr should be ns/2
ns = 640;     %  size of the square background
nr = 320;     %  eval theta at nr radial locations

% -- Physical parameters are hard-coded
Bi = 10;
Fomax = 0.2;
alpha = 1;        %  thermal diffusivity, order of magnitude of ceramic, m^2/s
r0 = 2.5e-2;                 %  radius of sphere, m
tmax = Fomax*r0^2/alpha;     %  time corresponding to max Fo

% --- Get analytical solution
Fo = linspace(0,alpha*tmax/r0^2,ntstep);
T = Tsphere(Bi,Fo,nr);

% --- Overwrite initial time with IC to eliminate any error
[m,nt] = size(T);    %  nt should equal ntstep, but we're being safe
T(:,1) = ones(m,1);  %  First column is at t=0, which should match IC

ibeg = 1 + fix(ns/2 - nr);    %  Center image of the sphere vertically
jbeg = 1 + fix(ns/2 - nr);    %  and horizontally 

% --- Define colormap range (not the map) to match the range of temperatures
cmin = min(min(T));
cmax = max(max(T));
caxis([cmin cmax]);
background = 0.5*(cmin+cmax);
S = background*ones(ns,ns);      %  Background color matrix

nf = 0;        %  Frame counter. Allows for loop to run backward if necessary
for k=1:nt

  C = colorCircle(T(:,k),background);     %  Generate color image of T(r) 
  [m,n] = size(C);                        %  m should equal n should equal nc
  S(ibeg:ibeg+m-1,jbeg:jbeg+n-1) = C;     %  Replace sphere image in S

  if fastRender
    imagesc(S,[cmin cmax]);    %  Draws color image with colormap [cmin cmax]
  else
    pcolor(S);
    shading('interp');        %  Bilinear interpolation from corners of cells
  end
  axis('square');  colormap(cmap);
  set(gca,'Xtick',[],'Ytick',[]); %  get rid of tick marks on the axes
  colorbar;  drawnow;

  if k==1
    M = moviein(nt);  %  preallocate memory for movie to improve performance
  end
  nf = nf + 1;        %  separate counter allows k to be traversed in any order
  M(:,nf) = getframe;

end

if nargout<1      %  show the movie if user doesn't want to save it
  movie(M,2);
else
  theMovie = M;
end

end
